local base_path = "/ssd/ml-1m/";
local output_path = "/output/ml-1m/caser/";
local loo_path = base_path + "loo/";
local max_seq_length = 200;
local metrics =  {
    mrr: [1, 5, 10],
    recall: [3, 5, 10],
    ndcg: [3, 5, 10]
};

local file_prefix = 'ml-1m';

{
    templates: {
        unified_output: {
            path: output_path
        },
        pos_neg_data_sources: {
            parser: {
                item_column_name: "title",
                features: {
                    user: {
                        type: "str",
                        column_name: "userId"
                    }
                }
            },
            loader: {
                batch_size: 128,
                max_seq_length: max_seq_length,
                num_workers: 4
            },
            path: base_path,
            train_file_prefix: file_prefix,
            validation_file_prefix: file_prefix,
            test_file_prefix: file_prefix,
            split_type: "leave_one_out"
        }
    },
    module: {
        type: "caser",
        metrics: {
            full: {
                metrics: metrics
            },
            sampled: {
                sample_probability_file: loo_path + file_prefix + ".popularity.title.txt",
                num_negative_samples: 100,
                metrics: metrics
            },
            random_negative_sampled: {
                metrics: metrics,
                num_negative_samples: 100
            }
        },
        model: {
            dropout: 0.5,
            max_seq_length: max_seq_length,
            embedding_size: 50,
            num_vertical_filters: 4,
            num_horizontal_filters: 16,
            fc_activation_fn: "relu",
            conv_activation_fn: "relu",
        }
    },
    tokenizers: {
        item: {
            tokenizer: {
                special_tokens: {
                    pad_token: "<PAD>",
                    mask_token: "<MASK>",
                    unk_token: "<UNK>"
                },
                vocabulary: {
                    delimiter: "\t",
                    file: loo_path + file_prefix + ".vocabulary.title.txt"
                }
            }
        },
        user: {
            tokenizer: {
                special_tokens: {
                    pad_token: "<PAD>",
                    mask_token: "<MASK>",
                    unk_token: "<UNK>"
                },
                vocabulary: {
                    delimiter: "\t",
                    file: loo_path + file_prefix + ".vocabulary.userId.txt"
                }
            }
        }
    },
    trainer: {
        loggers: {
            tensorboard: {}
        },
        checkpoint: {
            monitor: "recall@10_sampled(100)",
            save_top_k: 3,
            mode: 'max'
        },
        max_epochs: 800,
        check_val_every_n_epoch: 10,
        gpus: 1
    }
}