local base_path = "/ssd/ml-20m/";
local loo_path = base_path + "loo/";
local output_path = "/output/ml-20m/narm";
local max_seq_length = 200;
local metrics =  {
    mrr: [10],
    recall: [10],
    ndcg: [10]
};

local file_prefix = 'ml-20m';

{
    templates: {
        unified_output: {
            path: output_path
        },
        next_sequence_step_data_sources: {
            parser: {
                item_column_name: "title"
            },
            loader: {
                batch_size: 256,
                max_seq_length: max_seq_length,
                num_workers: 4,
                dynamic_padding: true
            },
            path: base_path,
            train_file_prefix: file_prefix,
            validation_file_prefix: file_prefix,
            test_file_prefix: file_prefix,
            split_type: "leave_one_out" // leave one out
        }
    },
    module: {
        type: "narm",
        metrics: {
            full: {
                metrics: metrics
            },
            sampled: {
                sample_probability_file: loo_path + file_prefix + ".popularity.title.txt",
                num_negative_samples: 100,
                metrics: metrics
            },
            random_negative_sampled: {
                num_negative_samples: 100,
                metrics: metrics
            },
        },
        model: {
            item_embedding_size: 50,
            global_encoder_size: 100,
            global_encoder_num_layers: 1,
            embedding_dropout: 0.25,
            context_dropout: 0.50,
            batch_first: true
        }
    },
    tokenizers: {
        item: {
            tokenizer: {
                special_tokens: {
                    pad_token: "<PAD>",
                    mask_token: "<MASK>",
                    unk_token: "<UNK>"
                },
                vocabulary: {
                    delimiter: "\t",
                    file: loo_path + file_prefix + ".vocabulary.title.txt"
                }
            }
        }
    },
    trainer: {
        loggers: {
            tensorboard: {}
        },
        checkpoint: {
            monitor: "recall@10_sampled(100)",
            save_top_k: 3,
            mode: 'max'
        },
        max_epochs: 100,
        gpus: 1,
    }
}
