local base_path = "/ssd/steam/";
local loo_path = base_path + "loo/";
local output_path = "/output/steam/gru";
local max_seq_length = 50;
local metrics =  {
    mrr: [1, 5, 10],
    recall: [3, 5, 10],
    ndcg: [3, 5, 10]
};

local file_prefix = 'preprocessed-steam';

{
    templates: {
        unified_output: {
            path: output_path
        },
        next_sequence_step_data_sources: {
            parser: {
                item_column_name: "product_id"
            },
            loader: {
                batch_size: 256,
                max_seq_length: max_seq_length,
                num_workers: 4
            },
            path: base_path,
            train_file_prefix: file_prefix,
            validation_file_prefix: file_prefix,
            test_file_prefix: file_prefix,
            split_type: "leave_one_out" // leave one out
        }
    },
    module: {
        type: "rnn",
        metrics: {
            full: {
                metrics: metrics
            },
            sampled: {
                sample_probability_file: loo_path + file_prefix + ".popularity.product_id.txt",
                num_negative_samples: 100,
                metrics: metrics
            },
             random_negative_sampled: {
                 metrics: metrics,
                 num_negative_samples: 100
             }
        },
        model: {
            cell_type: "gru",
            item_embedding_dim: 100,
            hidden_size: 100,
            num_layers: 1,
            dropout: 0.2
        }
    },
    tokenizers: {
        item: {
            tokenizer: {
                special_tokens: {
                    pad_token: "<PAD>",
                    mask_token: "<MASK>",
                    unk_token: "<UNK>"
                },
                vocabulary: {
                    delimiter: "\t",
                    file: loo_path + file_prefix + ".vocabulary.product_id.txt"
                }
            }
        }
    },
    trainer: {
        loggers: {
            tensorboard: {}
        },
        checkpoint: {
            monitor: "recall@10_sampled(100)",
            save_top_k: 3,
            mode: 'max'
        },
        max_epochs: 800,
        gpus: 1
    }
}