from dataclasses import dataclass
from pathlib import Path
from typing import List

ROOT_PATH = Path('/scratch/jane-doe-framework/experiments')


DEFAULT_NUM_EXAMPLES_LIST = [50, 100, 200, 300, 400, 500, 1000]


@dataclass
class Dataset:
    name: str
    sub_path: str
    latex_name: str
    experiment_id: str
    num_examples: List[int]
    data_file_name: str
    item_column_name: str
    user_column_name: str

    def normed_name(self) -> str:
        return self.name.lower().replace(' ', '_')


@dataclass
class Model:
    name: str
    sub_path: str


DATASETS = [
    Dataset('Amazon Beauty',
            'amazon/beauty',
            'ambeauty',
            'beauty',
            DEFAULT_NUM_EXAMPLES_LIST + [5000, 10000, 25000],
            'preprocessed-beauty.csv',
            'product_id',
            'reviewer_id'),
    Dataset('Amazon Games',
            'amazon/games',
            'amgame',
            'game',
            DEFAULT_NUM_EXAMPLES_LIST + [5000, 10000, 15000],
            'preprocessed-games.csv',
            'product_id',
            'reviewer_id'),
    Dataset('ML-1m', 'ml-1m/ml-1m', 'movielens', 'ml-1m', DEFAULT_NUM_EXAMPLES_LIST + [1500, 3000], 'ml-1m.csv', 'title', 'userId'),
    Dataset('ML-20m', 'ml-20m', 'movielenslarge', 'ml-2m', DEFAULT_NUM_EXAMPLES_LIST + [5000, 10000, 15000], 'ml-20m.csv', 'title', 'userId'),
    Dataset('Steam',
            'steam',
            'steam',
            'steam',
            DEFAULT_NUM_EXAMPLES_LIST + [2500, 5000, 10000],
            'preprocessed-steam.csv',
            'product_id',
            'username')
]


def get_dataset_by_name(name: str) -> Dataset:
    for dataset in DATASETS:
        if dataset.name == name:
            return dataset

    return None


MODELS = [
    Model('BERT4Rec', 'bert4rec'),
    Model('GRU', 'gru'),
    Model('NARM', 'narm'),
    Model('SASRec', 'sasrec')
]


def get_model_by_name(name: str) -> Model:
    for model in MODELS:
        if model.name == name:
            return model

    return None


# FIXME: some seeds are missing
SEEDS = [100987,
         107738,
         1097766,
         1098938,
         2734,
         382975,
         38399,
         388371,
         38999334,
         57847,
         58734948,
         7161632,
         7474,
         78987,
         84384509,
         902847,
         923934,
         92394378,
         92838389,
         9837]


