import json
import typer

from paper.common import ROOT_PATH, DATASETS


def _get_round(digits: int) -> str:
    return f'\\nprounddigits{{{digits}}}'


DEFAULT_ROUND = _get_round(2)
ACTION_ROUND = _get_round(1)


def format_number(value: int) -> str:
    return f'{value:,}'


def round_number(value: float, digits: int) -> str:
    return f'{value:.{digits}f}'


def main():
    use_siunitx = False

    print(f'% generated by {__file__}')
    print('\\begin{tabular}{lrrrrr}')
    print('\\toprule')
    print('Dataset & \\#users & \\#items & \\#actions & A. L. & Density \\\\')
    print('\\midrule')

    for dataset in DATASETS:
        result_file_path = ROOT_PATH / '../datasets' / dataset.sub_path / 'stats.json'

        with open(result_file_path) as result_file:
            stats = json.load(result_file)
            user_count = stats['user_count']
            item_count = stats['item_count']
            action_count = stats['action_count']
            action_count = action_count / 1000000  # in million
            avg_seq_length = stats['avg_seq_length']
            density = stats['density']

            if use_siunitx:
                print(f"\\{dataset.latex_name} & \\num{{{user_count}}} & \\num{{{item_count}}} & \\num[round-precision=1]{{{action_count}}}m & \\num[round-precision=2]{{{avg_seq_length}}} & \\num[round-precision=2]{{{density}}}\\% \\\\")
            else:
                print(f"\\{dataset.latex_name} & {format_number(user_count)} & {format_number(item_count)} & {round_number(action_count, 1)}m & {round_number(avg_seq_length, 2)} & {round_number(density, 2)}\\% \\\\")
    print('\\bottomrule')
    print('\\end{tabular}')


if __name__ == "__main__":
    typer.run(main)
