import json
from typing import Tuple

import pandas as pd
import typer

from paper.common import ROOT_PATH, DATASETS, Dataset


def _calc_stats(dataset: Dataset) -> Tuple[int, int, int, float, float]:
    df = pd.read_csv(ROOT_PATH / '../datasets' / dataset.sub_path / dataset.data_file_name, sep='\t')

    item_count = df[dataset.item_column_name].nunique()
    user_count = df[dataset.user_column_name].nunique()

    action_count = len(df.index)
    seq_lengths = df.groupby([dataset.user_column_name]).size()

    avg_seq_length = sum(seq_lengths.values) / seq_lengths.size

    density = avg_seq_length / item_count * 100
    return user_count, item_count, action_count, avg_seq_length, density


def main():
    for dataset in DATASETS:
        print(f'calculating stats for {dataset.name}')
        user_count, item_count, action_count, avg_seq_length, density = _calc_stats(dataset)
        result_file_path = ROOT_PATH / '../datasets' / dataset.sub_path / 'stats.json'

        with open(result_file_path, 'w') as result_file:
            json.dump({'user_count': user_count,
                       'item_count': item_count,
                       'action_count': action_count,
                       'avg_seq_length': avg_seq_length,
                       'density': density}, result_file)


if __name__ == "__main__":
    typer.run(main)
