import itertools
import json

from paper.common import MODELS, DATASETS, ROOT_PATH

if __name__ == '__main__':
    for dataset, model in itertools.product(DATASETS, MODELS):
        model_path = ROOT_PATH / dataset.sub_path / model.sub_path
        json_path = model_path / 'config_without_users.jsonnet'
        result_file = model_path / 'config_rank_experiment.json'
        with open(json_path) as json_file:
            json_config = json.load(json_file)
            trainer_config = json_config['trainer']
            # fix the output dirs
            log_path = str(model_path / 'predict_logs')
            trainer_config['checkpoint']['dirpath'] = log_path
            trainer_config['loggers']['tensorboard']['save_dir'] = log_path
            # set the gpu count
            gpu_count = 1
            trainer_config['gpus'] = gpu_count
            # add the new rank "metric" to the metrics config of the full/exact metric
            json_config['module']['metrics']['full']['metrics']['rank'] = []

            if gpu_count == 1:
                if 'accelerator' in trainer_config:
                    del trainer_config['accelerator']

            with open(result_file, 'w') as output_file:
                json.dump(json_config, output_file, indent=4)
