import itertools
from pathlib import Path

import typer
from zoller_python_utils.kubernetes.kubernetes_pipeline_util import Job, JobSet, RocketChatCallback, ConsoleLoggerCallback, execute_task, JobConfiguration

from paper.common import MODELS, get_dataset_by_name


def main(dataset_name: str):
    dataset = get_dataset_by_name(dataset_name)
    dataset_sub_path = dataset.sub_path
    subjobs = []

    normed_name = dataset.normed_name().replace('_', '-')

    for model, num_examples in itertools.product(MODELS, dataset.num_examples):
        model_name = model.name.lower()
        job_config_file = Path(f'../../../k8s/rank-experiment/{dataset_sub_path}/{model_name}/{num_examples}.yml').resolve()
        job_name = f'{normed_name}-{model_name}-{num_examples}-predict'
        job = Job(job_name, job_config_file)
        subjobs.append(job)

    rocket_chat_callback = RocketChatCallback.from_config()
    console_callback = ConsoleLoggerCallback()

    config = JobConfiguration('zoller', max_parallelism=40)
    execute_task(JobSet(f"{normed_name}", subjobs, parallel=True), config, callbacks= [console_callback, rocket_chat_callback])


if __name__ == "__main__":
    typer.run(main)
